#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <modbus.h>
#include <time.h>
#include <unistd.h>

#define SLAVE_ID 2
#define REGISTER_ADDRESS 0x0000
#define SERIAL_SPEED 19200
#define SERIAL_PARITY 'N'
#define SERIAL_DATABIT 8
#define SERIAL_STOPBIT 1

int modgus_communication(char *dev, int slave_id)
{
	time_t timer;
	modbus_t *ctx;
	int rc;
	int ret = EXIT_SUCCESS;
	uint16_t tab_rp_registers;

	ctx = modbus_new_rtu(dev, SERIAL_SPEED, SERIAL_PARITY,
					SERIAL_DATABIT, SERIAL_STOPBIT);

	if (ctx == NULL) {
		fprintf(stderr, "Unable to allocate libmodbus context\n"
				"modbus error\n");
		return EXIT_FAILURE;
	}

	modbus_set_debug(ctx, TRUE);

	modbus_set_error_recovery(ctx,
					MODBUS_ERROR_RECOVERY_LINK |
					MODBUS_ERROR_RECOVERY_PROTOCOL );

	if (modbus_set_slave(ctx, slave_id) == -1) {
		fprintf(stderr, "set slave failed: %s\n"
				"modbus error\n",
				modbus_strerror(errno));
		modbus_free(ctx);
		return EXIT_FAILURE;
	}

	if (modbus_connect(ctx) == -1) {
		fprintf(stderr, "Connection failed: %s\n"
				"modbus error\n",
					modbus_strerror(errno));

		modbus_free(ctx);
		return EXIT_FAILURE;
	}

	sleep(1);
	time(&timer);

	rc = modbus_read_input_registers(ctx, REGISTER_ADDRESS,
					2, &tab_rp_registers);

	if ( rc < 0 ) {
		fprintf(stderr, "faild: %s\n"
						"modbus error\n",
						modbus_strerror(errno));
		ret = EXIT_FAILURE;
	}

	printf("%s Temperature:%d\n\n", ctime(&timer), tab_rp_registers);


	modbus_close(ctx);

	modbus_free(ctx);

	return ret;
	}

	int main(int argc, char *argv[])
	{
	int ret;
	int slave_id = 1;

	if (argc < 2) {
		printf("Usage: %s <device>\n\n", argv[0]);
		return EXIT_SUCCESS;
	}

	while (1) {
		ret = modgus_communication(argv[1], slave_id);
		if (ret != EXIT_SUCCESS) {
				break;
		}

		if (slave_id == 2) {
			slave_id = 1;
		} else {
			slave_id++;
		}
	}

	return ret;
}
